require "./commit"

module TimeCost

  class Range
    class MissingOverlapError < RuntimeError ; end

    property time_start
    property time_stop
    property commits : Array(Commit)
    property author

    EPSILON_DEFAULT = 0.5

    def initialize(commit : Commit, @epsilon = EPSILON_DEFAULT)
      # FIXME: First approximation for users
      # later, we'll replace with @user = User.parse(commit.author)
      @author = commit.author

      @time_stop = commit.datetime 
      @time_start = commit.datetime - epsilon.hours
      @commits = [commit]
      self
    end

    def author : Author?
      @commits.first.try &.author 
    end

    def add(commit : Commit) : Range
    end

    def merge!(other_range) : Range
      raise MissingOverlapError.new if @time_stop < other_range.time_start
      raise MissingOverlapError.new if @time_start > other_range.time_stop

      #  B ------[----]----
      #  A  --[----]-------
      #  = ---[-------]----

      # boundaries
      new_start = [@time_start, other_range.time_start].min
      new_end = [@time_stop, other_range.time_stop].max

      @time_start = new_start
      @time_stop = new_end
      @commits.concat other_range.commits
      self
    end

    def merge(other_range) : Range
      copy = self.dup
      copy.commits = self.commits.dup

      copy.merge!(other_range)
    end

    def overlap?(range) : Bool
      result = false

      # return early result if ranges come from different authors
      return false if (@author != range.author)

      # Ref ----[----]-----
      # overlapping :
      #  A  -[----]--------
      #  B  -------[----]--
      #  C  -[----------]--
      #  D  ------[]-------
      # non-overlapping :
      #  E  -[]------------
      #  F  -----------[]--

      start_before_start = (range.time_start < @time_start)
      start_after_start = (range.time_start >= @time_start)
      start_after_stop = (range.time_start >= @time_stop)
      start_before_stop = (range.time_start < @time_stop)

      stop_before_stop = (range.time_stop < @time_stop)
      stop_after_stop = (range.time_stop >= @time_stop)
      stop_before_start = (range.time_stop < @time_start)
      stop_after_start = (range.time_stop >= @time_start)

      # A case
      if (start_before_start && start_before_stop &&
         stop_after_start && stop_before_stop)
        result = true
      end

      # B case
      if (start_after_start && start_before_stop &&
         stop_after_start && stop_after_stop)
        result = true
      end

      # C case
      if (start_before_start && start_before_stop &&
         stop_after_start && stop_after_stop)
        result = true
      end

      # D case
      if (start_after_start && start_before_stop &&
         stop_after_start && stop_before_stop)
        result = true
      end

      return result
    end

    def diff_hours : Float
      return ((@time_stop - @time_start) / (60* 60)).to_f
    end

    def to_s(show_authors = true) : String
      result = String.build do |val|
        val << "(%.2f)\t%s - %s\n" % [diff_hours, @time_start, @time_stop]
        if show_authors
          val << "\tby %s\n" % @commits.first.author.to_s
        end
        @commits.each do |commit|
          lines = commit.message.split(/\n/)
          r = lines.map_with_index do |line,i| 
            x = (i == 0) ? "[#{commit.commit_hash[0..7]}]" : " " * 9
            "\t#{x} #{line}"
          end.join("\n")
          val << r + "\n"
        end
      end
      result
    end
  end
end
